#!/bin/bash
# A user-centric script for Bogofilter training from maildirs
#
# Copyright Joe Stroller 2003 stroller at bigfoot dot come
# and distributed under the GNU Public License

# Set up variables
MAILDIR="$HOME/.Maildir"
BOGODIR="$HOME/.bogofilter"

# This is the spam corpus for training Bogofilter.
# With this default you'll probably want to have a Junk/Maybe folder, too,
# into which your MTA filters bogus-looking emails. That way you can run this
# script as a cron job & just drag & drop from "maybe" to "definite" in your
# mail-client to improve Bogofilter's accuracy.
SPAMDIR="$MAILDIR/.Junk.Definite"

# The read messages in all other mail folders are used as ham to train
# Bogofilter, EXCEPT FOR these we want to exclude:
HAMEXCLUDEFOLDERS="*List* *Sent* *Trash* *Junk*"
   # You MUST exlude your spam & deleted items folders!
   # I choose also to exclude my mailing lists, as they're filtered separately
   # and are not representative of my ham corpus. You may wish to change this.

# Saves me some typing, copying & pasting later
for folder in $HAMEXCLUDEFOLDERS
do
  HAMEXCLUDEARGS="$HAMEXCLUDEARGS -not -path $folder"
done

if [ ! -e $BOGODIR/wordlist.db ]
then
  if ( [ -e $BOGODIR/goodlist.db ] || [ -e $BOGODIR/spamlist.db ] )
  then 
    echo  "ERROR! This script requires Bogofilter >= v0.14 & uses a single wordlist file!"
    exit 1  
  else
      echo "No Bogofilter wordlist found, so BogoDB will be built from scratch"
      FINDDATEARGS=""
  fi
else
    # Determine when wordlist was last modified & format it suitably for `find`
    let DBAGE="$(date +%s) - $(date +%s -r $BOGODIR/wordlist.db)"
    let DBAGE="$DBAGE / 60"

    echo "Bogofilter DB was last updated $(date -R -d "$DBAGE minutes ago")"

    FINDDATEARGS="-mmin -$DBAGE"
fi

# NITTY-GRITTY - HERE'S THE BONES OF THE SCRIPT
# Find all mail in sub-folders of Inbox, dropping special & excluded folders.
# Inbox itself is ignored, as are unread messages, in case bogofilter has missed
# a spam coming in - we don't want to then train Bogofilter that that's ham!
# This script only treats as ham those emails you've read and filed away in a
# subfolder - that's the only way to be sure they're valid non-spam.

# NB: From `man maildir`: "Folders are additional subdirectories in the
#     maildir whose names begin with a period: such as .Drafts or .Sent."
#     So we first drop anything NOT beginning with a period & then an
#     alphanummeric (eg: courierimap*, cur, new & tmp.)

echo "Scanning ham email folders for new messages..." 
find $MAILDIR -path "$MAILDIR/.[[:alpha:]]*/cur/*" \
   $HAMEXCLUDEARGS $FINDDATEARGS \
   -print0 | xargs -0 bogofilter -n -W -v -B

echo "Scanning spam email folder for new messages..."
     
find $MAILDIR -path "$SPAMDIR/cur/*" $FINDDATEARGS \
   -print0 | xargs -0 bogofilter -s -W -v -B
exit 0


